==> socat-1.1.0.0.tar.bz2: socat-1.1/xio-listen.c <==
static const char _rcsid_[]="$Id: xio-listen.c,v 1.14 2002/05/30 14:45:28 gerhard Exp $";
/* Copyright Gerhard Rieger 2001, 2002 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* this file contains the source for listen socket options */

#include "xiosysincludes.h"
#include "xioopen.h"

#include "xio-named.h"
#include "xio-ip.h"
#include "xio-ip4.h"
#include "xio-listen.h"

#if WITH_LISTEN

/***** LISTEN addresses *****/
const struct optdesc opt_backlog = { "backlog",   NULL, OPT_BACKLOG,     GROUP_LISTEN, PH_LISTEN, TYPE_INT,    OFUNC_SPEC };
const struct optdesc opt_fork    = { "fork",      NULL, OPT_FORK,        GROUP_CHILD,   PH_PASTACCEPT, TYPE_BOOL,  OFUNC_SPEC };
/**/
#if (WITH_UDP || WITH_TCP)
const struct optdesc opt_range   = { "range",     NULL, OPT_RANGE,       GROUP_RANGE,  PH_ACCEPT, TYPE_STRING, OFUNC_SPEC };
#endif

int _xioopen_listen(struct single *fd, struct sockaddr *us, socklen_t uslen,
		 struct opt *opts, int pf, int socktype, int proto) {
   struct sockaddr sa;
   socklen_t salen;
   int backlog = 5;	/* why? 1 seems to cause problems under some load */
   bool dofork = false;
   pid_t pid;	/* mostly int; only used with fork */
   char *rangename;	/* allowed client address range - text form */
#if WITH_IP4
   byte4_t netaddr_in = 0;
   byte4_t netmask_in = 0;
#endif /* WITH_IP4 */
   char infobuff[256];
   char lisname[256];
   int result;

   retropt_bool(opts, OPT_FORK, &dofork);
#if 1
   if (dofork) {
      if (Signal(SIGCHLD, childdied) == SIG_ERR) {
	 Warn2("signal(SIGCHLD, %p): %s", childdied, strerror(errno));
      }
   }
#endif

   if ((fd->fd = Socket(pf, socktype, proto)) < 0) {
      Error4("socket(%d, %d, %d): %s", pf, socktype, proto, strerror(errno));
#if 0
      if (forever || retry) {
	 Sleep(intervall);
	 continue;
      }
#endif
      return STAT_RETRYLATER;
   }
   applyopts(fd->fd, opts, PH_PASTSOCKET);

   applyopts_cloexec(fd->fd, opts);

   applyopts(fd->fd, opts, PH_BIND);
   if (Bind(fd->fd, (struct sockaddr *)us, uslen) < 0) {
      Error4("bind(%d, {%s}, "F_Zd"): %s", fd->fd,
	     sockaddr_info(us, infobuff, sizeof(infobuff)), uslen,
	     strerror(errno));
      return STAT_RETRYLATER;
   }
#if WITH_UNIX
   if (us->sa_family == AF_UNIX) {
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_FD);
   }
#endif
   /* under some circumstances (e.g., TCP listen on port 0) bind() fills empty
      fields that we want need to know. */
   salen = sizeof(sa);
   if (Getsockname(fd->fd, us, &uslen) < 0) {
      Error4("getsockname(%d, %p, {%d}): %s",
	     fd->fd, &us, uslen, strerror(errno));
   }

   applyopts(fd->fd, opts, PH_PASTBIND);
#if WITH_UNIX
   if (us->sa_family == AF_UNIX) {
     /*applyopts_early(((struct sockaddr_un *)us)->sun_path, opts);*/
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_EARLY);
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_PREOPEN);
   }
#endif /* WITH_UNIX */

   retropt_int(opts, OPT_BACKLOG, &backlog);
   if (Listen(fd->fd, backlog) < 0) {
      Error3("listen(%d, %d): %s", fd->fd, backlog, strerror(errno));
      return STAT_RETRYLATER;
   }

#if WITH_IP4 /*|| WITH_IP6*/
   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
      parserange(rangename, us->sa_family, &netaddr_in, &netmask_in);
   }
#endif

   if (xioopts.logopt == 'm') {
      Info("starting accept loop, switching to syslog");
      diag_set('y', xioopts.syslogfac);  xioopts.logopt = 'y';
   } else {
      Info("starting accept loop");
   }
   Notice1("listening on %s", sockaddr_info(us, lisname, sizeof(lisname)));
   while (true) {	/* but we only loop if fork option is set */
      char peername[256];
      char sockname[256];
      int ps;		/* peer socket */
      char _peername[SOCKADDR_MAX];
      union sockaddr_union _sockname;
      struct sockaddr *pa = (struct sockaddr *)_peername;	/* peer address */
      union sockaddr_union *la = &_sockname;	/* local address */
      socklen_t pas = sizeof(_peername);	/* peer address size */
      socklen_t las = sizeof(_sockname);	/* local address size */
      salen = sizeof(struct sockaddr);

      do {
	 ps = Accept(fd->fd, (struct sockaddr *)&sa, &salen);
	 if (ps >= 0)  break;
	 if (errno == EINTR) {
	    continue;
	 }
	 if (errno == ECONNABORTED) {
	    Notice4("accept(%d, %p, {"F_Zu"}): %s",
		    fd->fd, &sa, salen, strerror(errno));
	    continue;
	 }
	 Error4("accept(%d, %p, {"F_Zu"}): %s",
		fd->fd, &sa, salen, strerror(errno));
	 return STAT_RETRYLATER;
      } while (true);
      applyopts_cloexec(ps, opts);
      if (Getpeername(ps, pa, &pas) < 0) {
	 Warn4("getpeername(%d, %p, {"F_socklen"}): %s",
	       ps, pa, pas, strerror(errno));
      }
      if (Getsockname(ps, &la->soa, &las) < 0) {
	 Warn4("getsockname(%d, %d, {"F_socklen"}): %s",
	       ps, pa, pas, strerror(errno));
      }
      Notice2("accepted connection from %s on %s",
	      sockaddr_info(pa, peername, sizeof(peername)),
	      sockaddr_info(&la->soa, sockname, sizeof(sockname)));

#if WITH_IP4
      if (checkrange(pa, &netaddr_in, &netmask_in) < 0) {
	 unsigned char infobuff[256];
	 Error1("refusing connection from %s",
		sockaddr_info((struct sockaddr *)pa,
			      infobuff, sizeof(infobuff)));
	 if (Shutdown(ps, 2) < 0) {
	    Error2("shutdown(%d, 2): %s", pa, strerror(errno));
	 }
	 continue;
      }
#endif /* WITH_IP4 */

      applyopts(fd->fd, opts, PH_FD);

      applyopts(fd->fd, opts, PH_CONNECTED);

      if (dofork) {
	 if ((pid = Fork()) < 0) {
	    Error1("fork(): %s", strerror(errno));
	    return STAT_RETRYLATER;
	 }
	 if (pid == 0) {	/* child */
	    if (Close(fd->fd) < 0) {
	       Warn2("close(%d): %s", fd->fd, strerror(errno));
	    }
	    fd->fd = ps;
	    break;
	 }
	 /* server: continue loop with listen */
	 /* shutdown() closes the socket even for the child process, but
	    close() does what we want */
	 if (Close(ps) < 0) {
	    Warn2("close(%d): %s", ps, strerror(errno));
	 }
	 Notice1("forked off child process "F_pid, pid);
	 Info("still listening");
      } else {
	 if (Close(fd->fd) < 0) {
	    Warn2("close(%d): %s", fd->fd, strerror(errno));
	 }
	 fd->fd = ps;
	break;
      }
   }

   if ((result = _xio_openlate(fd, opts)) < 0)
      return result;

   return 0;
}

#endif /* WITH_LISTEN */

==> socat-1.1.0.1.tar.bz2: socat-1.1/xio-listen.c <==
static const char _rcsid_[]="$Id: xio-listen.c,v 1.15 2002/06/08 14:54:18 gerhard Exp $";
/* Copyright Gerhard Rieger 2001, 2002 */
/* Published under the GNU General Public License V.2, see file COPYING */

/* this file contains the source for listen socket options */

#include "xiosysincludes.h"
#include "xioopen.h"

#include "xio-named.h"
#include "xio-ip.h"
#include "xio-ip4.h"
#include "xio-listen.h"

#if WITH_LISTEN

/***** LISTEN addresses *****/
const struct optdesc opt_backlog = { "backlog",   NULL, OPT_BACKLOG,     GROUP_LISTEN, PH_LISTEN, TYPE_INT,    OFUNC_SPEC };
const struct optdesc opt_fork    = { "fork",      NULL, OPT_FORK,        GROUP_CHILD,   PH_PASTACCEPT, TYPE_BOOL,  OFUNC_SPEC };
/**/
#if (WITH_UDP || WITH_TCP)
const struct optdesc opt_range   = { "range",     NULL, OPT_RANGE,       GROUP_RANGE,  PH_ACCEPT, TYPE_STRING, OFUNC_SPEC };
#endif

int _xioopen_listen(struct single *fd, struct sockaddr *us, socklen_t uslen,
		 struct opt *opts, int pf, int socktype, int proto) {
   struct sockaddr sa;
   socklen_t salen;
   int backlog = 5;	/* why? 1 seems to cause problems under some load */
   bool dofork = false;
   pid_t pid;	/* mostly int; only used with fork */
   char *rangename;	/* allowed client address range - text form */
#if WITH_IP4
   byte4_t netaddr_in = 0;
   byte4_t netmask_in = 0;
#endif /* WITH_IP4 */
   char infobuff[256];
   char lisname[256];
   int result;

   retropt_bool(opts, OPT_FORK, &dofork);
#if 1
   if (dofork) {
      if (Signal(SIGCHLD, childdied) == SIG_ERR) {
	 Warn2("signal(SIGCHLD, %p): %s", childdied, strerror(errno));
      }
   }
#endif

   if ((fd->fd = Socket(pf, socktype, proto)) < 0) {
      Error4("socket(%d, %d, %d): %s", pf, socktype, proto, strerror(errno));
#if 0
      if (forever || retry) {
	 Sleep(intervall);
	 continue;
      }
#endif
      return STAT_RETRYLATER;
   }
   applyopts(fd->fd, opts, PH_PASTSOCKET);

   applyopts_cloexec(fd->fd, opts);

   applyopts(fd->fd, opts, PH_BIND);
   if (Bind(fd->fd, (struct sockaddr *)us, uslen) < 0) {
      Error4("bind(%d, {%s}, "F_Zd"): %s", fd->fd,
	     sockaddr_info(us, infobuff, sizeof(infobuff)), uslen,
	     strerror(errno));
      return STAT_RETRYLATER;
   }
#if WITH_UNIX
   if (us->sa_family == AF_UNIX) {
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_FD);
   }
#endif
   /* under some circumstances (e.g., TCP listen on port 0) bind() fills empty
      fields that we want to know. */
   salen = sizeof(sa);
   if (Getsockname(fd->fd, us, &uslen) < 0) {
      Error4("getsockname(%d, %p, {%d}): %s",
	     fd->fd, &us, uslen, strerror(errno));
   }

   applyopts(fd->fd, opts, PH_PASTBIND);
#if WITH_UNIX
   if (us->sa_family == AF_UNIX) {
     /*applyopts_early(((struct sockaddr_un *)us)->sun_path, opts);*/
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_EARLY);
      applyopts_named(((struct sockaddr_un *)us)->sun_path, opts, PH_PREOPEN);
   }
#endif /* WITH_UNIX */

   retropt_int(opts, OPT_BACKLOG, &backlog);
   if (Listen(fd->fd, backlog) < 0) {
      Error3("listen(%d, %d): %s", fd->fd, backlog, strerror(errno));
      return STAT_RETRYLATER;
   }

#if WITH_IP4 /*|| WITH_IP6*/
   if (retropt_string(opts, OPT_RANGE, &rangename) >= 0) {
      parserange(rangename, us->sa_family, &netaddr_in, &netmask_in);
   }
#endif

   if (xioopts.logopt == 'm') {
      Info("starting accept loop, switching to syslog");
      diag_set('y', xioopts.syslogfac);  xioopts.logopt = 'y';
   } else {
      Info("starting accept loop");
   }
   Notice1("listening on %s", sockaddr_info(us, lisname, sizeof(lisname)));
   while (true) {	/* but we only loop if fork option is set */
      char peername[256];
      char sockname[256];
      int ps;		/* peer socket */
      char _peername[SOCKADDR_MAX];
      union sockaddr_union _sockname;
      struct sockaddr *pa = (struct sockaddr *)_peername;	/* peer address */
      union sockaddr_union *la = &_sockname;	/* local address */
      socklen_t pas = sizeof(_peername);	/* peer address size */
      socklen_t las = sizeof(_sockname);	/* local address size */
      salen = sizeof(struct sockaddr);

      do {
	 ps = Accept(fd->fd, (struct sockaddr *)&sa, &salen);
	 if (ps >= 0)  break;
	 if (errno == EINTR) {
	    continue;
	 }
	 if (errno == ECONNABORTED) {
	    Notice4("accept(%d, %p, {"F_Zu"}): %s",
		    fd->fd, &sa, salen, strerror(errno));
	    continue;
	 }
	 Error4("accept(%d, %p, {"F_Zu"}): %s",
		fd->fd, &sa, salen, strerror(errno));
	 return STAT_RETRYLATER;
      } while (true);
      applyopts_cloexec(ps, opts);
      if (Getpeername(ps, pa, &pas) < 0) {
	 Warn4("getpeername(%d, %p, {"F_socklen"}): %s",
	       ps, pa, pas, strerror(errno));
      }
      if (Getsockname(ps, &la->soa, &las) < 0) {
	 Warn4("getsockname(%d, %d, {"F_socklen"}): %s",
	       ps, pa, pas, strerror(errno));
      }
      Notice2("accepted connection from %s on %s",
	      sockaddr_info(pa, peername, sizeof(peername)),
	      sockaddr_info(&la->soa, sockname, sizeof(sockname)));

#if WITH_IP4
      if (checkrange(pa, &netaddr_in, &netmask_in) < 0) {
	 unsigned char infobuff[256];
	 Error1("refusing connection from %s",
		sockaddr_info((struct sockaddr *)pa,
			      infobuff, sizeof(infobuff)));
	 if (Shutdown(ps, 2) < 0) {
	    Error2("shutdown(%d, 2): %s", pa, strerror(errno));
	 }
	 continue;
      }
#endif /* WITH_IP4 */

      applyopts(fd->fd, opts, PH_FD);

      applyopts(fd->fd, opts, PH_CONNECTED);

      if (dofork) {
	 if ((pid = Fork()) < 0) {
	    Error1("fork(): %s", strerror(errno));
	    return STAT_RETRYLATER;
	 }
	 if (pid == 0) {	/* child */
	    if (Close(fd->fd) < 0) {
	       Warn2("close(%d): %s", fd->fd, strerror(errno));
	    }
	    fd->fd = ps;
	    break;
	 }
	 /* server: continue loop with listen */
	 /* shutdown() closes the socket even for the child process, but
	    close() does what we want */
	 if (Close(ps) < 0) {
	    Warn2("close(%d): %s", ps, strerror(errno));
	 }
	 Notice1("forked off child process "F_pid, pid);
	 Info("still listening");
      } else {
	 if (Close(fd->fd) < 0) {
	    Warn2("close(%d): %s", fd->fd, strerror(errno));
	 }
	 fd->fd = ps;
	break;
      }
   }

   if ((result = _xio_openlate(fd, opts)) < 0)
      return result;

   return 0;
}

#endif /* WITH_LISTEN */
